@@ -9,7 +9,7 @@ module RSpec
     #
     # This allows us to provide rich metadata about each individual
     # example without adding tons of methods directly to the ExampleGroup
-    # that users may inadvertantly redefine.
+    # that users may inadvertently redefine.
     #
     # Useful for configuring logging and/or taking some action based
     # on the state of an example's metadata.
@@ -138,8 +138,17 @@ module RSpec
 
         # don't clone the example group because the new example
         # must belong to the same example group (not a clone).
+        #
+        # block is nil in new_metadata so we have to get it from metadata.
         Example.new(example_group, description.clone,
-                    new_metadata, new_metadata[:block])
+                    new_metadata, metadata[:block])
+      end
+
+      # @private
+      def update_inherited_metadata(updates)
+        metadata.update(updates) do |_key, existing_example_value, _new_inherited_value|
+          existing_example_value
+        end
       end
 
       # @attr_reader
@@ -178,6 +187,16 @@ module RSpec
         @example_group_class = example_group_class
         @example_block       = example_block
 
+        # Register the example with the group before creating the metadata hash.
+        # This is necessary since creating the metadata hash triggers
+        # `when_first_matching_example_defined` callbacks, in which users can
+        # load RSpec support code which defines hooks. For that to work, the
+        # examples and example groups must be registered at the time the
+        # support code is called or be defined afterwards.
+        # Begin defined beforehand but registered afterwards causes hooks to
+        # not be applied where they should.
+        example_group_class.examples << self
+
         @metadata = Metadata::ExampleHash.create(
           @example_group_class.metadata, user_metadata,
           example_group_class.method(:next_runnable_index_for),
@@ -219,8 +238,7 @@ module RSpec
       def run(example_group_instance, reporter)
         @example_group_instance = example_group_instance
         @reporter = reporter
-        hooks.register_global_singleton_context_hooks(self, RSpec.configuration.hooks)
-        RSpec.configuration.configure_example(self)
+        RSpec.configuration.configure_example(self, hooks)
         RSpec.current_example = self
 
         start(reporter)
@@ -445,15 +463,14 @@ module RSpec
       def finish(reporter)
         pending_message = execution_result.pending_message
 
-        reporter.example_finished(self)
         if @exception
-          record_finished :failed
           execution_result.exception = @exception
+          record_finished :failed
           reporter.example_failed self
           false
         elsif pending_message
-          record_finished :pending
           execution_result.pending_message = pending_message
+          record_finished :pending
           reporter.example_pending self
           true
         else
@@ -465,6 +482,7 @@ module RSpec
 
       def record_finished(status)
         execution_result.record_finished(status, clock.now)
+        reporter.example_finished(self)
       end
 
       def run_before_example
